@@ -1,12 +1,12 @@
-# (C) Copyright 2015-2016 Hewlett Packard Enterprise Development Company LP
+# (C) Copyright 2015-2017 Hewlett Packard Enterprise Development LP
 
+import argparse
 import glob
 import hashlib
 import imp
 import inspect
 import itertools
 import math
-import optparse
 import os
 import platform
 import re
@@ -20,6 +20,9 @@ import traceback
 import uuid
 
 import logging
+
+from numbers import Number
+
 log = logging.getLogger(__name__)
 
 
@@ -441,21 +444,18 @@ def get_hostname():
         return hostname
 
 
-def get_parsed_args():
-    parser = optparse.OptionParser()
-    parser.add_option('-c', '--clean', action='store_true', default=False, dest='clean')
-    parser.add_option('-v', '--verbose', action='store_true', default=False, dest='verbose',
-                      help='Print out stacktraces for errors in checks')
-    parser.add_option('-f', '--config-file', default=None, dest='config_file',
-                      help='Location for an alternate config rather than '
-                           'using the default config location.')
+def get_parsed_args(prog=None):
+    parser = argparse.ArgumentParser(prog=prog)
+    parser.add_argument('-c', '--clean', action='store_true', default=False, dest='clean')
+    parser.add_argument('-v', '--verbose', action='store_true', default=False, dest='verbose',
+                        help='Print out stacktraces for errors in checks')
+    parser.add_argument('-f', '--config-file', default=None, dest='config_file',
+                        help='Location for an alternate config rather than '
+                             'using the default config location.')
 
-    try:
-        options, args = parser.parse_args()
-    except SystemExit:
-        # Ignore parse errors
-        options, args = optparse.Values({'clean': False}), []
-    return options, args
+    options = parser.parse_known_args(sys.argv[1:])
+
+    return options
 
 
 def load_check_directory():
@@ -613,8 +613,12 @@ def initialize_logging(logger_name):
             # make sure the log directory is writable
             # NOTE: the entire directory needs to be writable so that rotation works
             if os.access(os.path.dirname(log_file), os.R_OK | os.W_OK):
-                file_handler = logging.handlers.RotatingFileHandler(
-                    log_file, maxBytes=LOGGING_MAX_BYTES, backupCount=1)
+                if logging_config['enable_logrotate']:
+                    file_handler = logging.handlers.RotatingFileHandler(
+                        log_file, maxBytes=LOGGING_MAX_BYTES, backupCount=1)
+                else:
+                    file_handler = logging.FileHandler(log_file)
+
                 formatter = logging.Formatter(log_format, log_date_format)
                 file_handler.setFormatter(formatter)
 
@@ -697,3 +701,18 @@ def get_collector_restart_interval():
     restart_interval = agent_config.get('collector_restart_interval')
     restart_interval_in_sec = restart_interval * 60 * 60
     return restart_interval_in_sec
+
+
+def rollup_dictionaries(dict_sum, data):
+    # Add dictionaries. For example:
+    # data = {u'tx_dropped': 0, u'rx_packets': 18862, u'name': u'eth0',
+    # u'rx_bytes': 3808418, u'tx_errors': 0, u'rx_errors': 0,
+    # u'tx_bytes': 6100418, u'rx_dropped': 0, u'tx_packets': 60747}".
+    # When adding up dictionaries, only the keys with number type values can be
+    # added. Non-number type key value pairs should be ignored such as
+    # u'name': u'eth0'.
+    dict_sum = {k: dict_sum.get(k, 0) + data.get(k, 0)
+                for k in set(data) | set(dict_sum) if
+                isinstance(data.get(k, 0), Number) and
+                isinstance(dict_sum.get(k, 0), Number)}
+    return dict_sum
